/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: carol@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: CountryActivator.java 2214 2008-11-05 08:49:48Z eyindanga $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.example.deployment.country;

import java.io.File;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.ow2.cmi.config.CMIConfig;
import org.ow2.cmi.config.CMIConfigurator;
import org.ow2.cmi.config.ProtocolConfig;
import org.ow2.cmi.info.CMIInfoExtractor;
import org.ow2.cmi.info.CMIInfoExtractorException;
import org.ow2.cmi.info.CMIInfoRepository;
import org.ow2.cmi.info.ClusteredObjectInfo;
import org.ow2.cmi.test.GabonClass;
import org.ow2.cmi.test.GabonItf;
import org.ow2.util.execution.ExecutionResult;
import org.ow2.util.execution.IExecution;
import org.ow2.util.execution.helper.RunnableHelper;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Activator for the country Bean bundle.
 * @author eyindanga
 *
 */
public class CountryActivator implements BundleActivator {
    /**
     * Configuration directory.
     */
    private File confDir = null;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(CountryActivator.class);

    /**
     * Starts the bundle.
     *
     * @param bundleContext
     *            the bundle context.
     * @throws Exception
     *             any.
     */
    public void start(final BundleContext bundleContext) throws Exception {
        // Get the rmi port property
        final String confFileName = System.getProperty(CountryProperty.FILE_NAME_PROP.getPropertyName(),
                CountryProperty.DEFAULT_FILE_NAME_VALUE.getPropertyName());
        // Use integer value
        logger.info("Starting deployment of country Bean");
        IExecution<Void> registryExec = new IExecution<Void>() {

            public Void execute() throws Exception {

                confDir = new File(System.getProperty(CountryProperty.DIR_PROP.getPropertyName()));

                File configFile = new File(confDir.getAbsolutePath(), confFileName);
                CMIConfig<?> cmiConfig = CMIConfigurator.extractCMIConfig(configFile.toURL());
                // Gets the protocol.
                ProtocolConfig protocolConfig = (ProtocolConfig) cmiConfig.getJNDIConfig().getProtocolConfigs().toArray()[0];
                Hashtable<String, String> env = new Hashtable<String, String>();
                env.put(Context.INITIAL_CONTEXT_FACTORY, CountryProperty.CMI_CONTEXT_FACTORY.getPropertyName());
                env.put(CountryProperty.PROVIDER_URL_PROP.getPropertyName(), protocolConfig.getLocalRegistryUrl());

                bindObjectFromDD(CountryProperty.DEFAULT_OBJECT_NAME.getPropertyName(),
                        CountryProperty.DEFAULT_DD_NAME.getPropertyName());
                Context ctxt = new InitialContext(env);
                logger.info("Context retrieved.");
                GabonClass gabon = new GabonClass();
                ctxt.rebind(CountryProperty.DEFAULT_OBJECT_NAME.getPropertyName(), gabon);

                return null;
            }
        };
        // Execute
        RunnableHelper<Void> nsHelper = new RunnableHelper<Void>();

        ExecutionResult<Void> nsStartExecResult = nsHelper.execute(CMIConfigurator.class.getClassLoader(), registryExec);

        if (nsStartExecResult.hasException()) {
            logger.error("Unable to start CMI ", nsStartExecResult
                    .getException());
            return;
        }
    }

    /**
     * Stop the bundle.
     *
     * @param bundleContext
     *            the bundle context.
     * @throws Exception
     *             any.
     */
    public void stop(final BundleContext bundleContext) throws Exception {
        confDir = null;

    }

    /**
     * Bind a clustered object which infos are get from Deployment Descriptor.
     *
     * @param objectName
     *            the object name
     * @param descriptorFilename
     *            The deployment descriptor
     * @throws CMIInfoExtractorException
     *             any.
     */
    final void bindObjectFromDD(final String objectName,
            final String descriptorFilename) throws CMIInfoExtractorException {
        File descriptorFile = new File(confDir, descriptorFilename);
        try {
            ClusteredObjectInfo clusteredObjectInfoFromDD = CMIInfoExtractor.extractClusteringInfoFromDD(objectName, GabonItf.class,
                            null, descriptorFile.toURL(), false, false, null);
            CMIInfoRepository.addClusteredObjectInfo(objectName, clusteredObjectInfoFromDD);
        } catch (Exception e) {
            descriptorFile = null;
            throw new CMIInfoExtractorException(e.getMessage());
        }

    }

}
